﻿using System.Collections.Generic;
using System.Diagnostics;
using DotPrologTest.Execution;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Execution;
using OpcLabs.DotProlog.InputOutput;
using OpcLabs.DotProlog.Parsing;
using OpcLabs.DotProlog.Terms;
using OpcLabs.DotProlog.Test;

namespace DotPrologTest.Predicates.TermUnification
{
    [TestClass]
    // ReSharper disable InconsistentNaming
    public class UnifyWithOccursCheck_2_Test : SemiDeterministicPredicateTest
    // ReSharper restore InconsistentNaming
    {
        public UnifyWithOccursCheck_2_Test()
            : base("unify_with_occurs_check", 2)
        {
        }

        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Test_8_2_2_4_Examples_1()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse("'unify_with_occurs_check'(1, 1). "), Success(), "Expected success.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_2()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("'unify_with_occurs_check'(X, 1). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedX = new PInteger(1);
                var expected = new VariableDictionary { { variableNameDictionary["X"], expectedX } };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_3()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("'unify_with_occurs_check'(X, Y). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedX = variableNameDictionary["Y"];
                var expected = new VariableDictionary { { variableNameDictionary["X"], expectedX } };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_4()
        {
            using (IEngine engine = new Engine())
            {
                var readTermParser = new ReadTermParser();
                TextStream textStream = TextStream.Create("'unify_with_occurs_check'(_, _). ");
                var parsingContext = new ParsingContext();
                TermParsingError error = readTermParser.TryParse(textStream, ParsingParameters.Default, parsingContext, out Term term);
                Debug.Assert(error == null);
                Debug.Assert(term != null);

                var expected = new VariableDictionary { { parsingContext.VariableList[0], parsingContext.VariableList[1] } };
                Test(engine, term, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_5()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("'unify_with_occurs_check'(X, Y), 'unify_with_occurs_check'(X, abc). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedX = Atom.Add("abc");
                Term expectedY = Atom.Add("abc");
                var expected = new VariableDictionary
                {
                    { variableNameDictionary["X"], expectedX },
                    { variableNameDictionary["Y"], expectedY }
                };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_6()
        {
            using (IEngine engine = new Engine())
            {
                Term goal = Parse("'unify_with_occurs_check'(f(X, def), f(def, Y)). ", out IDictionary<string, Variable> variableNameDictionary);
                Term expectedX = Atom.Add("def");
                Term expectedY = Atom.Add("def");
                var expected = new VariableDictionary
                {
                    { variableNameDictionary["X"], expectedX },
                    { variableNameDictionary["Y"], expectedY }
                };
                Test(engine, goal, Success(expected));
                //FrontEnd.PrintResults(engine.Call(goal), variableNameDictionary);
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_7()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse("'unify_with_occurs_check'(1, 2). "), Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_8()
        {
            using (IEngine engine = new Engine())
            {
                Test(engine, Parse("'unify_with_occurs_check'(1, 1.0). "), Failure(), "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_9()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine, 
                    Parse(
                        @"'unify_with_occurs_check'( g(X), " +
                        @"     f(f(X)) ). "), 
                    Failure(), 
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_10()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine, 
                    Parse(
                        @"'unify_with_occurs_check'( f(X,  1), " +
                        @"     f(a(X) ) ). "), 
                    Failure(), 
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_11()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine, 
                    Parse(
                        @"'unify_with_occurs_check'( f(X,    Y,    X   ), " +
                        @"     f(a(X), a(Y), Y, 2) ). "), 
                    Failure(), 
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_12()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine, 
                    Parse(
                        @"'unify_with_occurs_check'( X, " +
                        @"     a(X) ). "),
                    Failure(),
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_13()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"'unify_with_occurs_check'( f(X,    1), " +
                        @"     f(a(X), 2) ). "),
                    Failure(),
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_14()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine, 
                    Parse(
                        @"'unify_with_occurs_check'( f(1, X,    1), " +
                        @"     f(2, a(X), 2) ). "),
                    Failure(),
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_15()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"'unify_with_occurs_check'( f(1,    X), " +
                        @"     f(2, a(X)) ). "),
                    Failure(),
                    "Expected failure.");
            }
        }

        [TestMethod]
        public void Test_8_2_2_4_Examples_16()
        {
            using (IEngine engine = new Engine())
            {
                Test(
                    engine,
                    Parse(
                        @"'unify_with_occurs_check'( f(X,    Y,    X, 1), " +
                        @"     f(a(X), a(Y), Y, 2) ). "),
                    Failure(),
                    "Expected failure.");
            }
        }

        // ReSharper restore InconsistentNaming
    }
}
