﻿
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Threading;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;

namespace OpcLabs.UnitTestLib
{
    static public class AssertUtilities
    {
        #region Static Public Methods

        static public void AreEqualInfos<TInfo>(TInfo expectedInfo, TInfo actualInfo) where TInfo : Info
        {
            if (ReferenceEquals(expectedInfo, null))
            {
                Assert.IsNull(actualInfo);
                return;
            }

            bool equals = expectedInfo.Equals(actualInfo, out List<string> annotationList);
            Assert.IsNotNull(annotationList);
            Assert.IsTrue(equals, String.Concat(((IEnumerable<string>)annotationList).Reverse()));
        }

        static public void BeginWriteLine()
        {
            Trace.Write("## ");
        }

        static public void Done()
        {
            WriteLine();
            WriteLine("Done.");
        }

        static public void EndWriteLine()
        {
            Trace.WriteLine("");
        }

        // some test runners do not show the message passed to Inconclusive by themselves
        [TerminatesProgram]
        static public void Inconclusive([NotNull] string message)
        {
            WriteLine($"Inconclusive: {message}");
            Assert.Inconclusive(message);
        }

        // some test runners do not show the message passed to Inconclusive by themselves
        [TerminatesProgram]
        static public void Inconclusive([NotNull] string message, [NotNull] params object[] parameters)
        {
            Inconclusive(String.Format(CultureInfo.CurrentCulture, message, parameters));
        }

        static public void Phase([NotNull] string caption)
        {
            WriteLine();
            WriteLine($"Phase: {caption}...");
        }

        static public void WaitingPhase(int milliseconds)
        {
            Phase($"Waiting for {milliseconds} milliseconds");
            Thread.Sleep(milliseconds);
        }

        static public void WriteLine()
        {
            WriteLine("");
        }

        static public void WriteLine([CanBeNull] object value)
        {
            BeginWriteLine();
            Trace.WriteLine((value?.ToString() ?? ""));
        }

        static public void WriteLine([NotNull] string message)
        {
            BeginWriteLine();
            Trace.WriteLine(message);
        }

        /// <exclude/>
        static public void WriteSeparatorLine()
        {
            WriteLine(new string('-', 71));
        }

        #endregion
    }
}
