﻿
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.OperationModel;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA
{
    public partial class EasyUAClientTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [DynamicData(nameof(UALiveTestEnvironment.OpcFoundation_UASampleServer_OptimisticSupportedEndpointDescriptorDataRows), typeof(UALiveTestEnvironment))]
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void WriteTest_OpcFoundation_SampleServer_SourceAndServerTimestamp_NotSupported(
            [NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var target = new EasyUAClient();
            UANodeId nodeId = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_UserScalar_Int32Value;
            try
            {
                target.Write(endpointDescriptor, nodeId,
                    new UAAttributeData(12345, UASeverity.GoodOrSuccess, DateTime.UtcNow, DateTime.UtcNow /* writing server timestamp is not supported by the sample server*/));
            }
            catch (UAException uaException)
            {
                Exception innerException = uaException.InnerException;
                Assert.IsNotNull(innerException);
                var serviceException = innerException as UAServiceException;
                Assert.IsNotNull(serviceException);
                Assert.AreEqual(UACodeBits.BadWriteNotSupported, serviceException.ServiceResult.StatusCode.CodeBits);
                return;
            }
            Assert.Fail("No exception");
        }

        [DynamicData(nameof(UALiveTestEnvironment.OpcFoundation_UASampleServer_OptimisticSupportedEndpointDescriptorDataRows), typeof(UALiveTestEnvironment))]
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void WriteTest_OpcFoundation_SampleServer_SourceTimestamp([NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var target = new EasyUAClient();
            UANodeId nodeId = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_UserScalar_Int32Value;

            TestSection.Act();
            target.Write(endpointDescriptor, nodeId,
                new UAAttributeData(12345, UASeverity.GoodOrSuccess, DateTime.UtcNow/*, DateTime.UtcNow: writing server timestamp is not supported by the sample server*/));
            // IMPROVE: test whether it was written
        }

        [DynamicData(nameof(UALiveTestEnvironment.OpcFoundation_UASampleServer_OptimisticSupportedEndpointDescriptorDataRows), typeof(UALiveTestEnvironment))]
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void WriteValueTest_OpcFoundation_SampleServer([NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var target = new EasyUAClient();
            UANodeId nodeId = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_UserScalar_Int32Value;

            TestSection.Act();
            target.WriteValue(endpointDescriptor, nodeId, 1);
            // IMPROVE: test whether it was written
        }

        [DynamicData(nameof(UALiveTestEnvironment.OpcFoundation_UASampleServer_OptimisticSupportedEndpointDescriptorDataRows), typeof(UALiveTestEnvironment))]
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void WriteValueTest_OpcFoundation_SampleServer_Array([NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var target = new EasyUAClient();
            UANodeId nodeId = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_UserArray_UInt16Value;

            target.WriteValue(endpointDescriptor, nodeId,
                new UInt16[] { 123, 456, 789 }, typeof(UInt16[]));
            // IMPROVE: test the outcome            
        }

        [DynamicData(nameof(UALiveTestEnvironment.OpcFoundation_UASampleServer_OptimisticSupportedEndpointDescriptorDataRows), typeof(UALiveTestEnvironment))]
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void WriteValueTest_OpcFoundation_SampleServer_Int16ToInt32Value([NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            var target = new EasyUAClient();
            UANodeId nodeId = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_Scalar_Int32Value;

            TestSection.Act();
            target.WriteValue(endpointDescriptor, nodeId, (Int16)1);
            // IMPROVE: test whether it was written
        }

        // ReSharper restore InconsistentNaming
        #endregion



        #region public Methods

        #endregion
    }
}
